import java.io.*;

public class Simulator {

    private int[] M; // Datenspeicher
    private int[] P; // Programmspeicher
    private int[] R; // Register
    private int PC; // Program Counter: Befehlszaehler

    // Befehle als OpCodes
    static final int ADD  = 0;  // Ra = Rb + Rc
    static final int SUB  = 1;  // Ra = Rb - Rc
    static final int MUL  = 2;  // Ra = Rb * Rc
    static final int DIV  = 3;  // Ra = Rb / Rc
    static final int MOD  = 4;  // Ra = Rb % Rc
    static final int CMP  = 5;  // Ra = SIGNUM(Rb - Rc)

    static final int NOT  = 6;  // Ra = NOT Rb = ~Rb
    static final int AND  = 7;  // Ra = Rb AND Rc = Rb & Rc
    static final int OR   = 8;  // Ra = Rb OR Rc = Rb | Rc
    static final int XOR  = 9;  // Ra = Rb XOR Rc = Rb ^ Rc
    static final int SH   = 10; // ARITHMETIC SHIFT: Ra = Rb SHIFTED BY Rc

    static final int ADDI = 11; // Ra = Rb + c
    static final int SUBI = 12; // Ra = Rb - c
    static final int MULI = 13; // Ra = Rb * c
    static final int DIVI = 14; // Ra = Rb / c
    static final int MODI = 15; // Ra = Rb % c
    static final int CMPI = 16; // Ra = SIGNUM(Rb - c)

    static final int ANDI = 17; // Ra = Rb AND c = Rb & c
    static final int ORI  = 18; // Ra = Rb OR c = Rb | c
    static final int XORI = 19; // Ra = Rb XOR c = Rb ^ c
    static final int SHI  = 20; // ARITHMETIC SHIFT: Ra = Rb SHIFTED BY c

    static final int LDO  = 21; // LOAD +OFFSET: Ra = MEM[Rb + c]
    static final int LDX  = 22; // LOAD +INDEX: Ra = MEM[Rb + Rc]
    static final int STO  = 23; // STORE +OFFSET: MEM[Rb + c] = Ra
    static final int STX  = 24; // STORE +INDEX: MEM[Rb + Rc] = Ra

    static final int BEQ  = 25; // BRANCH TO INSTRUCTION +/- b IF Ra IS EQUAL TO ZERO
    static final int BNE  = 26; // BRANCH TO INSTRUCTION +/- b IF Ra IS NOT EQUAL TO ZERO
    static final int BLT  = 27; // BRANCH TO INSTRUCTION +/- b IF Ra IS LESS THAN ZERO
    static final int BGE  = 28; // BRANCH TO INSTRUCTION +/- bIF Ra IS GREATER THAN OR EQUAL TO ZERO
    static final int BLE  = 29; // BRANCH TO INSTRUCTION +/- b IF Ra IS LESS THAN OR EQUAL TO ZERO
    static final int BGT  = 30; // BRANCH TO INSTRUCTION +/- b IF Ra IS GREATER THAN ZERO
    static final int JSR  = 31; // JUMP TO INSTRUCTION a, SAVE PC+1 IN REGISTER 31
    static final int RET  = 32; // JUMP TO INSTRUCTION Ra, HALT IF a = 0

    static final int RDD  = 33; // READ DECIMAL NUMBER FROM INPUT INTO Ra
    static final int WRD  = 34; // WRITE DECIMAL NUMBER FROM Ra TO OUTPUT
    static final int WRH  = 35; // WRITE NUMBER FROM Ra TO OUTPUT IN HEXADECIMAL NOTATION
    static final int WRL  = 36; // WRITE NEWLINE TO OUTPUT

    private Simulator(String filename) {


		// Arrays, Register und Programmzaehler initialisieren


        Parser.parse(filename, P); // Laedt das Programm aus der Datei in den Programmspeicher
    }

    /**
     * Startet die Simulation des Assemblers.
     *
     * @param args[0] Dateiname einer Datei mit Assembler-Code
     * @param args[1] Dateiname einer Datei mit Werten (optional)
     * @param args[2] Offset, ab dem die Werte in den Datenspeicher geladen werden (optional, Standard: 1000)
     */
    public static void main(String[] args) {
        if (args.length >= 1) {
            Simulator sim = new Simulator(args[0]);

            if (args.length >= 2) {
                int offset = 1000;
                if (args.length >= 3) offset = Integer.parseInt(args[2]);
                Parser.load(args[1], sim.M, offset);
            }

            sim.execute();
        }
    }

    private void execute() {


		// Assemblerprogramm ausfuehren


    }

    /**
     * Liest einen Wert von der Tastatur ein.
     *
     * @return einen auf der Tastatur eingegebenen int-Wert
     */
    private int input() {
        int ret = 0;
        try {
            BufferedReader rdd = new BufferedReader(new InputStreamReader(System.in));
            System.out.print("Input: ");
            ret = Integer.parseInt(rdd.readLine());
         } catch (Exception e) {
             System.err.println("Fehler bei der Tastatureingabe:");
             System.err.println(e.getMessage());
             System.exit(1);
         }
         return ret;
    }

}
